<?php 
use Automattic\WooCommerce\Utilities\OrderUtil;
use Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController;

class WCCM_OrderDetailsPage
{
	public function __construct()
	{
		add_action( 'add_meta_boxes', array( &$this, 'add_meta_boxes' ) );
		add_action( 'woocommerce_admin_order_data_after_order_details',  array( &$this,'display_custom_data' ));
	}
	public function add_meta_boxes()
	{
		$screen = 'shop_order';
		try
		{
			$screen = wc_get_container()->get( CustomOrdersTableController::class )->custom_orders_table_usage_is_enabled()
			? wc_get_page_screen_id( 'shop-order' )
			: 'shop_order';
		}
		catch (Exception $e) {$screen = 'shop_order';}
		
		add_meta_box( 'woocommerce-customers-manager-user-note', esc_html__('User notes', 'woocommerce-customers-manager'), array( &$this, 'add_user_note_meta_box' ), $screen, 'side', 'high');
		
	}
	public function add_user_note_meta_box($post_or_order_object)
	{
		global $wcuf_option_model, $wcuf_upload_field_model, $wcuf_media_model, $wccm_customer_model;
		$order = ( $post_or_order_object instanceof WP_Post ) ? wc_get_order( $post_or_order_object->ID ) : $post_or_order_object;
		$order_id = ( $post_or_order_object instanceof WP_Post ) ? $post_or_order_object->ID : $post_or_order_object->get_id();
		$user_id = $order->get_customer_id();
		
		if($user_id == 0)
			return;
		
		?>
			<p><?php echo $wccm_customer_model->get_user_notes($user_id); ?></p>
			<a class="button-primary" target="_blank" href="<?php echo get_admin_url()."admin.php?page=woocommerce-customers-manager&customer={$user_id}&action=customer_details"; ?>"> <?php esc_html_e('Edit', 'woocommerce-customers-manager' ); ?> </a>
		<?php
	}
	public function display_custom_data( $order ) 
	{
		global $wccm_order_model;
		$user_id = $order->get_customer_id();
		
		$user_details_page_link = $user_id != 0 ? get_admin_url()."admin.php?page=woocommerce-customers-manager&customer={$user_id}&action=customer_details" : 
												  get_admin_url()."admin.php?page=woocommerce-customers-manager&customer_email={$order->get_billing_email()}&action=customer_details";
		
		$is_guest = false;
		if($user_id != 0)
		{
			$order_number = count($wccm_order_model->get_user_orders_ids($user_id));
		}
		else 
		{
			$is_guest = true;
			$order_number = $wccm_order_model->get_guest_orders(false, $order->get_billing_email(), true);
		}
		
		wp_enqueue_style('wcccm-order-details', WCCM_PLUGIN_PATH.'/css/admin-order-details-page.css'); 
		?>
		<div class="wccm_order_details_column">
			<p class="form-field form-field-wide">
			  <label class="wcccm_label"><?php esc_html_e('First time customer:', 'woocommerce-customers-manager') ?></label>
			  <?php echo $order_number > 1 ? esc_html__('No', 'woocommerce-customers-manager') : esc_html__('Yes', 'woocommerce-customers-manager'); ?>		  
			</p>
			<p class="form-field form-field-wide">
			  <label class="wcccm_label"><?php esc_html_e('Number of orders: ', 'woocommerce-customers-manager') ?></label>
			  <?php echo $order_number; 
			  if($is_guest): ?>
			  <small> <?php esc_html_e('(orders with same billing email address)', 'woocommerce-customers-manager') ?></small>
			  <?php endif; ?>
			</p>		
			<?php do_action( 'wccm_order_details_page_after_customer_details', $order_number, $user_id); ?>
			<a target="_blank" id="wccm_user_details_button" class="button-primary wccm_user_datails_button" href="<?php echo $user_details_page_link; ?>"> <?php esc_html_e('Details & Orders list', 'woocommerce-customers-manager') ?></a>
		</div>
		<?php
	}
}
?>