<?php
class WCCM_Order
{
	var $start_date;
	var $end_date;
	var $min_amount;
	var $max_amount;
	var $min_amount_total;
	var $max_amount_total;
	var $statuses;
	var $per_page;
	var $page_num;
	var $customer_ids;
	var $product_ids;
	var $category_ids;
	var $product_relationship;
	var $product_category_relationship;
	var $product_category_filters_relationship;
	var $billing_fields = array('billing_first_name', 'billing_last_name', 'billing_company', 'billing_address_1', 'billing_address_2', 'billing_city', 'billing_state', 'billing_postcode', 'billing_country', 'billing_email', 'billing_phone', 'formatted_billing_address');
	var $shipping_fields = array('shipping_first_name', 'shipping_last_name', 'shipping_company', 'shipping_address_1', 'shipping_address_2', 'shipping_city', 'shipping_state', 'shipping_postcode', 'shipping_country', 'formatted_shipping_address');
	var $special_fields = array('shipping_tax'); 
	
	public function __construct()
	{
		if(is_admin())
		{
			add_action('wp_ajax_wccm_get_orders_data', array(&$this, 'ajax_get_orders'));
			add_action('wp_ajax_wccm_get_orders_tot_num', array(&$this, 'ajax_get_tot_orders_num'));
			add_action('wp_ajax_wccm_get_order_list', array(&$this, 'ajax_get_order_list'));
		}
		//add_action('woocommerce_admin_order_data_after_payment_info', array(&$this, 'test'));
		
	}
	public function test()
	{
		$languages =  apply_filters( 'wpml_active_languages', NULL, 'skip_missing=0orderby=id&order=desc' );
		wccm_var_dump($languages);
	}
	public static function get_order_id($order)
	{
		return version_compare( WC_VERSION, '2.7', '<' ) ? $order->id : $order->get_id();
	}
	public static function get_billing_email($order)
	{
		if(version_compare( WC_VERSION, '2.7', '<' ))
			return $order->billing_email;
		
		return $order->get_billing_email();
	}
	public function update_order_customer_id($user_id, $email, $order_id = null)
	{
		$orders = wc_get_orders(
				array(
						'limit' => -1,
						'billing_email' => $email,
						'id'  	=> $order_id,
					)		
		);
		
		if($orders)
			foreach($orders as $order)
			{
				$order->set_customer_id($user_id);
				$order->save();
			}
	}
	public function update_order_download_permission($order_id, $user_id, $email)
	{
		$data_store = WC_Data_Store::load( 'customer-download' );
		$data_store->update_user_by_order_id($order_id, $user_id, $email);
	}
	public function get_order_months($post_type = 'shop_order' )
	{
		global $wpdb;
		 $months = $wpdb->get_results( "
            SELECT DISTINCT YEAR( date_created_gmt ) AS year, MONTH( date_created_gmt ) AS month
            FROM {$wpdb->prefix}wc_orders
            ORDER BY date_created_gmt DESC");
		
		return $months;
	}
	public function ajax_get_order_list()
	{
		global $wccm_customer_model;
		if(!$wccm_customer_model->is_shop_manager())
		{
			echo 0;
			wp_die();
		}
		
		$resultCount = 50;
		$search_string = isset($_GET['search_string']) ? $_GET['search_string'] : null;
		$page = isset($_GET['page']) ? $_GET['page'] : null;
		$offset = isset($page) ? ($page - 1) * $resultCount : null;
		$orders = $this->get_order_list($search_string ,$offset, $resultCount);
		 echo json_encode( $orders);
		 wp_die();
	}
	public function ajax_get_orders()
	{
		global $wccm_customer_model;
		if(!$wccm_customer_model->is_shop_manager())
		{
			echo 0;
			wp_die();
		}
		
		$orders_array = array();
		$this->get_request_parameters();
		
		$customers = $this->get_customer_ids_from_orders(false);
		
		echo json_encode($customers);
		
		wp_die(); 
	
	}
	public function ajax_get_tot_orders_num()
	{
		global $wccm_customer_model;
		if(!$wccm_customer_model->is_shop_manager())
		{
			echo 0;
			wp_die();
		}
		
		$this->get_request_parameters();
		$result =  $this->get_orders_tot_num();
		
		echo isset($result) ? $result:0;
		wp_die();
	}
	private function get_order_list($search_string = null, $offset = null, $resultCount  = null)
	{
		global $wpdb;
		$statuses = $this->get_order_statuses();
		if($search_string)
		{
			$offset = null;
			$limit_query = "";
		}
		
		//New: HPOS 
		$args = array(
			'status' => $statuses['statuses'],
			'return' => 'ids'
		);
		if($search_string)
		{
			$args['limit'] = -1;
			$orders = array();
			$results = array();
			$args['field_query'] = array(
				 'relation' => 'OR',
				 array(
					'field'   => 'billing_first_name',
					'value'   => $search_string,
					'compare' => 'LIKE',
					),
				array(
					'field'   => 'billing_last_name',
					'value'   => $search_string,
					'compare' => 'LIKE',
					),	
				array(
					'field'   => 'billing_email',
					'value'   => $search_string,
					'compare' => 'LIKE',
					),	
				 array(
					'field'   => 'shipping_first_name',
					'value'   => $search_string,
					'compare' => 'LIKE',
					),
				array(
					'field'   => 'shipping_last_name',
					'value'   => $search_string,
					'compare' => 'LIKE',
					),
				array(
					'field'   => 'customer_id',
					'value'   => $search_string,
					'compare' => 'LIKE',
					),
				array(
					'field'   => 'id',
					'value'   => $search_string,
					'compare' => 'LIKE',
					),
				);
			$results[] = wc_get_orders($args);
			
			unset($args['field_query']);
			$args['meta_query'] = 
			array(
				array(
				  'relation' => 'OR',
					  array(
						'key'     => 'order_number',
						'value'   => $search_string,
						'compare' => 'LIKE',
					  ),
					  array(
						'key'     => 'order_number_formatted',
						'value'   => $search_string,
						'compare' => 'LIKE',
					  ),
				)
			);
			
			$results[] = wc_get_orders($args);
			
			//For some reasons meta_query and field_query cannot be combiled 
			$results = array_filter($results);
			$orders = array_intersect(...$results);
		}
		else
		{
			$args['limit'] = $resultCount;
			$args['paged'] = $offset;
			$orders = wc_get_orders($args);
		}
		 					
		$bad_char = array('"', "'");
		$results = array();
		foreach((array)$orders as $key => $order_id)
		{
			$order = wc_get_order($order_id);
			$date = $order->get_date_created();
			$user = $order && $order->get_customer_id() > 0 ? get_userdata($order->get_customer_id()) : null;
			
			$results[$key] = new \stdClass;
			$results[$key]->order_id = $order->get_id();
			$results[$key]->order_date = $date->date_i18n();
			$results[$key]->order_status = $order->get_status();
			$results[$key]->order_number_formatted = $order->get_order_number();
			$results[$key]->order_number = $order->get_order_number();
			$results[$key]->billing_name_and_last_name = $order ? str_replace($bad_char, "", $order->get_billing_first_name()." ".$order->get_billing_last_name()) : "N/A";
			$results[$key]->shipping_name_and_last_name =  $order ? str_replace($bad_char, "",$order->get_shipping_first_name()." ".$order->get_shipping_last_name()) : "N/A";
			$results[$key]->user_login = isset($user) ? $user->user_login: "Guest";
			$results[$key]->user_id = $order ? $order->get_customer_id() : "N/A";
			$results[$key]->user_email =  $order ?  $order->get_billing_email() : "N/A";
		}
		
		if(isset($offset) && isset($resultCount))
		{
			$orders = wc_get_orders(array(
								"limit" => -1,
								'return'       => 'ids',
							)); 
			$num_order = $orders ? count($orders) : 0;
			$endCount = $offset + $resultCount;
			$morePages = $num_order > $endCount;
			$results = array(
				  "results" => $results,
				  "pagination" => array(
					  "more" => $morePages
				  )
			  );
		}
		else
			$results = array(
				  "results" => $results,
				  "pagination" => array(
					  "more" => $false
				  )
			  );
		return $results;
	}
	
	public function get_request_parameters()
	{
		$this->start_date = isset($_POST['start_date']) && strtotime($_POST['start_date']) !== false ? $_POST['start_date']." 00:00:01": null;
		$this->end_date = isset($_POST['end_date'])  && strtotime($_POST['end_date']) !== false  ? $_POST['end_date']." 23:59:59": null;
		$this->min_amount = filter_input( INPUT_POST, 'min_amount', FILTER_VALIDATE_INT ) ? $_POST['min_amount']: null;
		$this->max_amount = filter_input( INPUT_POST, 'max_amount', FILTER_VALIDATE_INT ) ? $_POST['max_amount']: null;
		$this->min_amount_total = filter_input( INPUT_POST, 'min_amount_total', FILTER_VALIDATE_INT ) ? $_POST['min_amount_total']: null;
		$this->max_amount_total = filter_input( INPUT_POST, 'max_amount_total', FILTER_VALIDATE_INT ) ? $_POST['max_amount_total']: null;
		$this->statuses = isset($_POST['statuses']) ? explode(",",$_POST['statuses']): null;	
		$this->per_page = filter_input( INPUT_POST, 'per_page', FILTER_VALIDATE_INT ) ? $_POST['per_page']: null;	
		$this->page_num = filter_input( INPUT_POST, 'page_num', FILTER_VALIDATE_INT ) ? $_POST['page_num']: null;	
		$this->customer_ids = isset($_POST['customer_ids']) && @$_POST['customer_ids'][0] != "" ? explode(",",$_POST['customer_ids']): null;	//Not used in SQL
		$this->product_ids = isset($_POST['product_ids']) && @$_POST['product_ids'][0] != "" ? explode(",",$_POST['product_ids']): null;		//Not used in SQL
		$this->category_ids = isset($_POST['category_ids']) && @$_POST['category_ids'][0] != "" ? explode(",",$_POST['category_ids']): null;	//Not used in SQL
		$this->product_relationship = isset($_POST['product_relationship']) && ($_POST['product_relationship'] == "and" || $_POST['product_relationship'] == "or") ? $_POST['product_relationship']: null;	
		$this->product_category_relationship = isset($_POST['product_category_relationship']) && ($_POST['product_category_relationship'] == "and" || $_POST['product_category_relationship'] == "or")  ? $_POST['product_category_relationship']: null;	
		$this->product_category_filters_relationship = isset($_POST['product_category_filters_relationship']) && ($_POST['product_category_filters_relationship'] == "and" || $_POST['product_category_filters_relationship'] == "or")  ? $_POST['product_category_filters_relationship']: null;	
	
		if(is_array($this->statuses) && !empty($this->statuses))
		{
			$statuses = $this->get_order_statuses();
			$error = false;
			foreach($this->statuses as $curr_status)
				if(!isset($statuses[$curr_status]))
					$error = true;
				
			if($error)
				$this->statuses = null;
		}
	}
	public function get_max_order_total()
	{
		$args = array(
			'limit'			=>	1,
			'orderby'       => 'total',
			'order'         => 'DESC',
		);
		$result = wc_get_orders($args);
		return $result && !empty($result) ? $result[0]->get_total() : 0;
	}
	public function get_user_total_spent($customer_email = null, $user_id = null)
	{
		$args = array(
			'limit'			=>	-1,
			'field_query' => array(
				array(
					'field'   => 'customer_id',
					'value'   => $user_id ? $user_id : 0,
					'compare' => $user_id ? "=" : '>',
				)
			)
		);
		if($customer_email)
		{
			$args['field_query']['relation'] = "AND";
			$args['field_query'][] = array(
					'field'   => 'billing_first_name',
					'value'   => $customer_email,
					'compare' => '=',
					);
		}
		$orders = wc_get_orders($args);
		$result = array();
		foreach($orders as $order)
		{
			if(!isset($result[$order->get_customer_id()]))
				$result[$order->get_customer_id()] = 0;
			
			$result[$order->get_customer_id()] += $order->get_total();
		}
		return !empty($result) ? max($result) : 0;
	}
	
	/* public function get_new_order_status($without_prefix = false, $get_codes = true)
	{
		$statuses = wc_get_order_statuses();
		$result = array();
		foreach($statuses as $code => $status_name)
		{
			$status_code = $without_prefix ? str_replace("wc-", "", $code) : $code;
			if($get_codes)
				$result[] = $status_code;
			else 
				$result[$code] = $status_name;
		}
	} */
	public function get_order_statuses($get_codes = true)
	{
		$result = array('version'=>0, 'statuses'=>array());
		if(function_exists( 'wc_get_order_statuses' ))
		{
			
			$result['version'] = 2.2;
			//[slug] => name
			if(!$get_codes)
				$result['statuses'] = wc_get_order_statuses();
			else foreach(wc_get_order_statuses() as $code => $name)
				$result['statuses'][] = $code;
		}
		else
		{
			$args = array(
				'hide_empty'   => false, 
				'fields'            => 'id=>name', 
			);
			$result['version'] = 2.1;
			//[id] => name
			$result['statuses'] =  get_terms('shop_order_status', $args);
		}
		return $result;
	}
	public function get_order_statuses_id_to_name()
	{
		$result = array();
		if(function_exists( 'wc_get_order_statuses' ))
		{
			
			$result['version'] = 2.2;
			//[slug] => name
			$result['statuses'] = wc_get_order_statuses();
		}
		else
		{
			$args = array(
				'hide_empty'   => false, 
				'fields'            => 'id=>name', 
			);
			$result['version'] = 2.1;
			//[id] => name
			$result['statuses'] =  get_terms('shop_order_status', $args);
		}
		return $result;
	}
	
	public function assign_users($order_ids, $user_id, $params = array())
	{
		if(!isset($order_ids) || !is_array($order_ids) || !isset($user_id))
			return;
		
		$billing_fields = array('billing_first_name', 'billing_last_name', 'billing_company', 'billing_address_1', 'billing_address_2', 'billing_city', 'billing_state', 'billing_postcode', 'billing_country', 'billing_email', 'billing_phone');
		$shipping_fields = array('shipping_first_name', 'shipping_last_name', 'shipping_company', 'shipping_address_1', 'shipping_address_2', 'shipping_city', 'shipping_state', 'shipping_postcode', 'shipping_country');
		foreach($order_ids as $order_id)
		{
			$order = wc_get_order($order_id);
			$order->set_customer_id($user_id);
			$customer = new WC_Customer($user_id);
			if($params['overwrite_billing_data'])
			{
				foreach ( $billing_fields as $key) 
				{
					if ( is_callable( array( $order, "set_{$key}" ) ) && is_callable( array( $customer, "get_{$key}" ) )  ) 
					{
						$method_name = "get_".$key;
						$order->{"set_{$key}"}( $customer->$method_name() );

					
					} 
				}
			}
			if($params['overwrite_shipping_data'])
			{
				foreach ( $shipping_fields as $key ) 
				{
					if ( is_callable( array( $order, "set_{$key}" ) ) && is_callable( array( $customer, "get_{$key}" ) ) ) 
					{
						$method_name = "get_".$key;
						$order->{"set_{$key}"}( $customer->$method_name());

					}
				}
			}
			$order->save();
		}
	}
	public function get_orders_query_conditions_to_exclude_bad_orders($join_type = 'INNER')
	{
		global $wpdb;
		$statuses = $this->get_order_statuses();
		$result = array();
		$result['join'] = "";
		$result['where'] = "";
		$result['version'] = $statuses['version'];
		if($statuses['version'] > 2.1)
		{
			$result['statuses'] = $statuses['statuses'] = array_diff($statuses['statuses'], array('wc-cancelled', 'wc-refunded', 'wc-failed'));
			$result['where'] = " AND posts.post_status IN ('".implode( "','",$statuses['statuses'])."') ";
		}
		else 
		{
			$result['statuses'] = $statuses['statuses'] = array_diff($statuses['statuses'], array('cancelled', 'refunded', 'failed'));
			$result['join'] = " {$join_type} JOIN {$wpdb->term_relationships} AS rel ON posts.ID=rel.object_id
							  {$join_type} JOIN {$wpdb->term_taxonomy} AS tax ON tax.term_taxonomy_id = rel.term_taxonomy_id
							  {$join_type} JOIN {$wpdb->terms} AS term ON term.term_id = tax.term_id ";
			$result['where'] .= " AND posts.post_status   = 'publish'
								 AND tax.taxonomy        = 'shop_order_status' 
								 AND term.slug           IN ( '" .implode( "','",$statuses['statuses']). "' )";
		}
		
		return $result;
	}
	public function get_status_for_old_orders($join_type = 'INNER')
	{
		global $wpdb;
		if(function_exists( 'wc_get_order_statuses' ))
			return false;
		$result = array();
		$result['select'] = ", term.slug as order_status";
		$result['join'] = " {$join_type} JOIN {$wpdb->term_relationships} AS rel ON posts.ID=rel.object_ID
				  {$join_type} JOIN {$wpdb->term_taxonomy} AS tax USING( term_taxonomy_id ) AND tax.taxonomy = 'shop_order_status'
				  {$join_type} JOIN {$wpdb->terms} AS term USING( term_id ) ";
				  
		return $result;
	}
	public function get_orders_tot_num( )
	{
		return $this->get_customer_ids_from_orders(true);
	}
	public function get_customer_ids_from_orders($select_count = false)
	{
		global $wpdb, $wccm_wpml_helper;
		$result = array('ids'=>array(),'emails'=>array());
		$all_product_ids_from_categories = array();
		$all_product_ids = array();
		$total_sale_range = "";
		$total_sale_user_range = "";
		$date_range = "";
		$status = "";
		$offset_limit = "";
		$customers_ids_joins = "";
		$customers = "";
		$products_ids_joins = "";
		$products = "";
		$lang_array = null;
		if(isset($this->min_amount))
			$total_sale_range .= " AND orders.total_amount >= ".$this->min_amount;
		if(isset($this->max_amount))
			$total_sale_range .= " AND orders.total_amount <= ".$this->max_amount;
		if(isset($this->min_amount_total) && isset($this->max_amount_total))
			$total_sale_user_range .= " HAVING (SUM(orders.total_amount) >= ".$this->min_amount_total." AND SUM(orders.total_amount) <= ".$this->max_amount_total.") ";
		if($this->start_date)
			$date_range .= " AND orders.date_created_gmt >= '".$this->start_date."'";
		if($this->end_date)
			$date_range .= " AND orders.date_created_gmt <= '".$this->end_date."'";
		if($this->statuses)
		{
				$status = " AND  orders.post_status IN ('".implode( "','",$this->statuses)."') ";
		}
		if(isset($this->page_num) && isset($this->per_page))
		{
			
			$offset = ($this->page_num-1)*$this->per_page;
			$offset_limit .=  " LIMIT {$offset},{$this->per_page} ";
		}
		if($this->customer_ids)
		{
			
			$customers= " AND orders.billing_email IN ('" . implode( "','", $this->customer_ids). "' )";
		}
		else //GUEST CUSTOMERS
		{
			
			$customers= " AND orders.customer_id != 1 "; //>1 NO GUEST
						
		}
		//PRODUCT AND PRODUCT CATEGORIES ---> WPML	
		$all_product_ids_from_categories = array();
		if($this->category_ids)
		{
			$all_category_ids = array();
			foreach($this->category_ids as $category_id)
			{
				$all_category_ids[] = $wccm_wpml_helper->get_all_product_id_translations($category_id, true, true);
			}
			//$this->var_debug($all_category_ids);
			foreach($all_category_ids as $category_ids_per_language)
			{
				$get_prdocuts_from_categories = " SELECT DISTINCT GROUP_CONCAT(products.ID) as product_ids
										 FROM {$wpdb->posts} AS products 
										 INNER JOIN {$wpdb->term_relationships} AS term_rel ON term_rel.object_id = products.ID
										 INNER JOIN {$wpdb->term_taxonomy} AS term_tax ON term_tax.term_taxonomy_id = term_rel.term_taxonomy_id 
										 INNER JOIN {$wpdb->terms} AS terms ON terms.term_id = term_tax.term_id
										 WHERE  terms.term_id IN ('" . implode( "','", $category_ids_per_language). "')  
										 AND term_tax.taxonomy = 'product_cat' GROUP BY terms.term_id ";
				$wpdb->query('SET group_concat_max_len=500000'); 
				$additional_products = $wpdb->get_results($get_prdocuts_from_categories);
				$additional_products = isset($additional_products) ? $additional_products:array("-1");
				array_push($all_product_ids_from_categories, $additional_products);
			}
			
		}
		if($this->product_ids)
		{
			$all_product_ids = array();
			foreach($this->product_ids as $product_id)
			{
				$all_product_ids[] = $wccm_wpml_helper->get_all_product_id_translations($product_id, false, true);
			}
		}
		if($this->product_ids || $this->category_ids)
		{
			$products_ids_joins = " INNER JOIN {$wpdb->prefix}woocommerce_order_items AS ordermeta_products ON ordermeta_products.order_id = orders.ID  
									INNER JOIN {$wpdb->prefix}woocommerce_order_itemmeta AS ordermeta_product_meta0 ON ordermeta_product_meta0.order_item_id = ordermeta_products.order_item_id ";
			
			$product_pre_condtions = " AND  (ordermeta_product_meta0.meta_key = '_product_id' OR ordermeta_product_meta0.meta_key = '_variation_id' ) ";
			$products .= " AND (";
		}	
		
		//products
		$counter = $index = 0;
		if($this->product_ids)	
		{
			$products .= " (";
			foreach($all_product_ids as $products_ids_and_translated_ids)
			{
				if($counter != 0)
				{
					if($this->product_relationship == 'and')
					{
					    $products_ids_joins .= " INNER JOIN {$wpdb->prefix}woocommerce_order_items AS ordermeta_products{$counter} ON ordermeta_products{$counter}.order_id = orders.ID  
											     INNER JOIN {$wpdb->prefix}woocommerce_order_itemmeta AS ordermeta_product_meta{$counter} ON ordermeta_product_meta{$counter}.order_item_id = ordermeta_products{$counter}.order_item_id ";
						$product_pre_condtions .= " AND  (ordermeta_product_meta{$counter}.meta_key = '_product_id' OR ordermeta_product_meta{$counter}.meta_key = '_variation_id' ) ";
						$index = $counter;
					}
					$products .= " {$this->product_relationship} " ;
				}
				$products .= "  ordermeta_product_meta{$index}.meta_value IN ('" .implode( "','", $products_ids_and_translated_ids). "' ) ";
				$counter++;
			}
			$products .= " ) ";
		}
		
		//categories
		$counter2 = $index = 0;
		if(isset($this->category_ids))
		{
			if(isset($this->product_ids))
				$products .= " {$this->product_category_filters_relationship} (";
			
			foreach($all_product_ids_from_categories as $products_per_category_per_lanuage)
			{
				if(!empty($products_per_category_per_lanuage))
				{
					if($counter != 0)
					{
						if($this->product_category_relationship == 'and')
						{
							$products_ids_joins .= " INNER JOIN {$wpdb->prefix}woocommerce_order_items AS ordermeta_products{$counter} ON ordermeta_products{$counter}.order_id = orders.ID  
													 INNER JOIN {$wpdb->prefix}woocommerce_order_itemmeta AS ordermeta_product_meta{$counter} ON ordermeta_product_meta{$counter}.order_item_id = ordermeta_products{$counter}.order_item_id ";
							$product_pre_condtions .= " AND  (ordermeta_product_meta{$counter}.meta_key = '_product_id' OR ordermeta_product_meta{$counter}.meta_key = '_variation_id' ) ";				
							$index = $counter;
						}
						
					}
					if($counter2 != 0)
						$products .= " {$this->product_category_relationship} ";
					
					//$products .= " ( ";
					$ids_temp = array();
					foreach($products_per_category_per_lanuage as $products_per_category)
					{
						$ids_temp = array_merge($ids_temp, explode(",", $products_per_category->product_ids));
						
						$counter2++;
					}
					$products .= "  ordermeta_product_meta{$index}.meta_value IN ('" . implode( "','", $ids_temp). "' ) ";
					
					$counter++;
				}
			}
			if(isset($this->product_ids))
				$products .= " ) ";
		}
			
		if($this->product_ids || $this->category_ids)
			$products .= " ) ";
		
		$wpdb->query('SET SQL_BIG_SELECTS=1');
		
		$select_type = "SELECT DISTINCT orders.id, orders.customer_id as customer_id, orders.billing_email as billing_email, SUM(orders.total_amount) as total_spent ";
		if($select_count)
			$select_type = "SELECT COUNT(DISTINCT orders.id) AS total_orders ";
		$query = $select_type.
				 "FROM {$wpdb->prefix}wc_orders as orders".
				 $products_ids_joins.
				 " WHERE 1=1 ".$total_sale_range.$date_range.$status.$customers.$products;
		
		//HPOS: custom query compatible
		if(!$select_count)
		{
			$query .="GROUP BY orders.billing_email ".$offset_limit ; //ordermeta_customer.meta_value
			$order_ids = $wpdb->get_results($query);
			
		}
		else
		{
			$query .= "GROUP BY orders.id ".$total_sale_user_range;
			$order_ids = count($wpdb->get_results($query));
		}
		
		if($select_count)
			return $order_ids;
		if(isset($order_ids))
			foreach($order_ids as $customer_orders_data)
			{ 
				
			 $total_spent = $customer_orders_data->customer_id != 0 ? $this->get_user_total_spent($customer_orders_data->billing_email):$this->get_user_total_spent(null, $customer_orders_data->customer_id); 
			  if((!isset($this->min_amount_total) && !isset($this->max_amount_total)) ||
			    ( (  $total_spent >= $this->min_amount_total ) && (   $total_spent <= $this->max_amount_total ) ))
				{
					array_push($result['emails'], $customer_orders_data->billing_email);
					array_push($result['ids'], $customer_orders_data->customer_id);
				}
				
			}
		return $result;  
		/* 
		//HPOS 
		global $wpdb, $wccm_wpml_helper;
		$args = array(
			'limit'			=> -1,
			'meta_query'	=> array('relation' => 'AND'),
			'field_query'	=> array('relation' => 'AND', 
										array(
											'field'   => 'customer_id',
											'value'   => 0,
											'compare' => '>',
											)
					), 
			
		);
		
		//Options
		if(isset($this->min_amount))
		{
			$args['field_query'][] = array(
												'field'		=> 'total',
												'value'		=>	$this->min_amount,
												'compare'	=> ">="
											);
		}
		if(isset($this->max_amount))
		{
			$args['field_query'][] = array(
												'field'		=> 'total',
												'value'		=>	$this->max_amount,
												'compare'	=> "<"
											);
		}
		if($this->statuses)
		{
			$args['status'] = $this->statuses;
		}
		if($this->start_date)
		{
			$args['date_query']['inclusieve']	= true;
			$args['date_query']['after'] = array('year'		=> date("Y",strtotime($this->start_date)),
												 'month'	=> date("m",strtotime($this->start_date)),
												 'day'		=> date("d",strtotime($this->start_date)),
												 'hour'		=> 00,
												 'minute'	=> 00,
												 'second'	=> 00
												);
		}
		if($this->end_date)
		{
			$args['date_query']['inclusieve']	= true;
			$args['date_query']['before'] = array('year'	=> date("Y",strtotime($this->end_date)),
												 'month'	=> date("m",strtotime($this->end_date)),
												 'day'		=> date("d",strtotime($this->end_date)),
												 'hour'		=> 00,
												 'minute'	=> 00,
												 'second'	=> 00
												);
		} 
		
		$orders = wc_get_orders($args);
		$result = array('ids'=>array(),'emails'=>array());
		$total_spent = array();
		
		//Post processing
		//Products and categories filter
		$products_ids = array();
		if($this->category_ids)
		{
			$all_category_ids = array();
			foreach($this->category_ids as $category_id)
			{
				$all_category_ids[] = $wccm_wpml_helper->get_all_product_id_translations($category_id, true, true);
			}
			
			foreach($all_category_ids as $category_ids_per_language)
			{
				$get_prdocuts_from_categories = " SELECT DISTINCT GROUP_CONCAT(products.ID) as product_ids
										 FROM {$wpdb->posts} AS products 
										 INNER JOIN {$wpdb->term_relationships} AS term_rel ON term_rel.object_id = products.ID
										 INNER JOIN {$wpdb->term_taxonomy} AS term_tax ON term_tax.term_taxonomy_id = term_rel.term_taxonomy_id 
										 INNER JOIN {$wpdb->terms} AS terms ON terms.term_id = term_tax.term_id
										 WHERE  terms.term_id IN ('" . implode( "','", $category_ids_per_language). "')  
										 AND term_tax.taxonomy = 'product_cat' GROUP BY terms.term_id ";
				$wpdb->query('SET group_concat_max_len=500000'); 
				$additional_products = $wpdb->get_results($get_prdocuts_from_categories);
				if(isset($additional_products))
					foreach($additional_products as $additional_product)
						$products_ids[] = $additional_product->product_ids;
			}
			
		}
		//ToDo: Relation AND OR
		if($this->product_ids)
		{
			foreach($this->product_ids as $product_id)
			{
				$products_ids = array_merge($products_ids, $wccm_wpml_helper->get_all_product_id_translations($product_id, false, true));
			}
		}
		if($products_ids)
		{
			$products_ids = array_unique($products_ids);
			$order_with_products = array();	
			foreach($products_ids as $product_id)
					$order_with_products = array_merge($order_with_products, $this->get_orders_by_product_id($product_id, 0, $orders));
			$orders = $order_with_products;
		}
			   
		
		//Retrieved order are filter by extracting ids and emails
		foreach($orders as $index => $order)	
		{
			if(!isset($total_spent[$order->get_customer_id()]))
				$total_spent[$order->get_customer_id()] = 0;
			$total_spent[$order->get_customer_id()] += $order->get_total();
			
			if(!in_array($order->get_customer_id(), $result['ids']))
				$result['ids'][$order->get_customer_id()] = $order->get_customer_id();
			if(!in_array($order->get_billing_email(), $result['emails']))
				$result['emails'][$order->get_customer_id()] = $order->get_billing_email();
		}
		//filter by total amount
		if(isset($this->min_amount_total) || isset($this->max_amount_total))
		 foreach($total_spent as $customer_id => $customer_total_spent)
		   {
				if((!isset($this->min_amount_total) || $customer_total_spent > $this->min_amount_total) && 
				(!isset($this->max_amount_total) || $customer_total_spent < $this->max_amount_total)) 
				{

				}	
				else
				{
				 unset($result['ids'][$customer_id]);
				 unset($result['emails'][$customer_id]);
				}						
		   }
		   
		
		//Pagination
		$start_index = ($this->page_num-1)*$this->per_page;
		$end_index = $start_index+$this->per_page;
		
		if($select_count)
			return count($result);
		
		
		return array_slice($result, $start_index, $this->per_page);
		*/
	}	
	public function get_guest_user_data_from_last_order($user_email)
	{
		global $wpdb;
		$order = wc_get_orders(
				array(
						'limit'			=>	1,
						'orderby'       => 'date',
						'order'         => 'DESC',
						'field_query'	=> array(
												'field'   => 'billing_email',
												'value'   => $user_email,
												'compare' => '=',
												)
					)		
		);
		$user_data = array('customer_info' => new \stdClass(), 'customer_extra_info' => array());
		$order = $order && isset($order[0]) ? $order[0] : false;
		if($order)
		{		
			$user_data['customer_info']->user_email = $order->get_billing_email();
			$user_data['customer_info']->user_registered = "N/A";
			$order_methods_names = array_merge($this->billing_fields, $this->shipping_fields, $this->special_fields); 
			foreach ( $order_methods_names as $field_name) 
			{
				$method_name = "get_".$field_name;
				if ( is_callable( array( $order, $method_name ) )  ) 
					$user_data['customer_extra_info'][$field_name][0] = $order->$method_name();
			}
		}
		
		return $user_data;
	}
	private function get_orders_by_product_id($product_id, $customer_id = -1, $orders = false)
	{
		//HPOS: full compliant
		$args = array(
						'limit'			=>	-1
					);
		
		if($orders === false)
		{
			if($customer_id >= 0)
				$args['customer_id'] = $customer_id;
			
			$orders = wc_get_orders($args);
		}
		$result = array();
		//ToDo: to improve. For now there isn't any method to filter via wc_get_orders() using product ids 
		if($orders)
			foreach($orders as $order)
			{
				$items = $order->get_items();
				foreach ( $items as $item ) 
				{
					$product_name = $item['name'];
					if($product_id == $item['product_id'] || $product_id == $item['variation_id'] )
					{
						$result[] = $order;
						break;
					}
				}
			}
			
		return $result;
	}
	public function get_guest_orders($filter_by_product = false, $filter_by_emails = false, $get_count = false, $current_page = null, $per_page = null)
	{
			 
		if($filter_by_product != false) 
		{
			//HPOS: custom query
			global $wpdb,$wccm_customer_model, $wccm_wpml_helper;
			$all_product_id = array();	
			$all_product_id = $wccm_wpml_helper->get_all_product_id_translations($filter_by_product, false, true);
			
			$join = " INNER JOIN {$wpdb->prefix}woocommerce_order_items AS wc_order_items ON wc_order_items.order_id = posts.ID	
					  INNER JOIN {$wpdb->prefix}woocommerce_order_itemmeta AS wc_order_item_meta ON wc_order_item_meta.order_item_id = wc_order_items.order_item_id	";
					  
			$where = "  AND (wc_order_item_meta.meta_key = '_product_id' OR wc_order_item_meta.meta_key = '_variation_id' ) 
						AND wc_order_items.order_item_type IN ( 'line_item' ) 
						AND (wc_order_item_meta.meta_value IN ('".implode("','",$all_product_id)."')) ";
		
		
			$query_string = "
				 SELECT      COUNT(DISTINCT posts.ID) as total_guest_orders
				 FROM        {$wpdb->prefix}wc_orders as orders
				 {$join}
				 WHERE     	 orders.customer_id = 0
				 {$where}
				";  
			$result = $wpdb->get_row($query_string, OBJECT );
			return $result->total_guest_orders;
		}
		else if($filter_by_emails != false)
		{
			$args['field_query'] = array(
				 'relation' => 'OR',
					 array(
						'field'   => 'billing_email',
						'value'   => $filter_by_emails,
						'compare' => 'LIKE',
						),
			);
			
		}//HPOS
		$args = array(
						'limit'			=> -1,
						'customer_id'	=> 0,
					);
					
		if($get_count) //To save memory
		{
			$args['return'] = 'ids';
		}
		elseif($current_page && $per_page)
		{
			$args['limit'] = $per_page;
			$args['paged'] = $current_page;
		} 
		$orders = wc_get_orders($args);
		return $get_count ? ($orders ? count($orders) : 0) : $orders;
	}
	public function get_vat_number_field_name()
	{
		global $wcev_order_model;
		return !isset($wcev_order_model) ? '_billing_vat_number' : '_billing_eu_vat';
	}
	public function get_order_details_by_id($order_id)
	{
		$order = wc_get_order( $order_id );
		$items = $order->get_items();
		return $items;
	}
	public function get_all_user_orders($user_id, $starting_date = null, $ending_date = null, $filter_by_product_id = null)
	{
		if(!isset($user_id))
			return false;
		
		$search_by_email = false; //seek for user orders using his email uses (post_meta.meta_key -> _billing_email)
		if(!is_numeric($user_id))
			$search_by_email = true;
		
		$version_and_statuses = $this->get_order_statuses();
		$tax_query = array();
		$orders = array();//order ids
		$statuses = array_keys($version_and_statuses['statuses']);
		
		$args['limit'] = -1;
		$orders = array();
		$results = array();
		$args['field_query'] = array('relation' => 'OR');	
		
		if(!$search_by_email)
		{
			$args['field_query'][] = array(
					'field'   => 'customer_id',
					'value'   => $user_id,
					'compare' => 'LIKE',
					);
		}
		else
		{
			$args['field_query'][] = array(
					'field'   => 'billing_email',
					'value'   => $user_id,
					'compare' => 'LIKE',
					);
		}
		if(isset($starting_date) && isset($ending_date) && !empty($starting_date) && !empty($starting_date))
		{
			list($starting_year, $starting_month, $starting_day) = explode ("/", $starting_date, 3);
			list($ending_year, $ending_month, $ending_day) = explode ("/", $ending_date, 3);
			
			$args['date_query']  = array( 'after' => array('year'  => $starting_year,
															'month' => $starting_month,
															'day'   => $starting_day,
															 'hour' => 00,
															 'minute' => 00,
															 'second' => 00
															),
										  'before' => array('year'  => $ending_year,
															'month' => $ending_month,
															'day'   => $ending_day,
															'hour' => 23,
															 'minute' => 59,
															 'second' => 59
															),
										  'inclusive' => true	
 										);
		}
		
		
		$orders = wc_get_orders($args); 	
		if($filter_by_product_id != null)
		{
			$new_orders_list = array();
			foreach($orders as $order)
			{
				$items = $order->get_items();
				foreach($items as $product)
				{
					if( $product['item_meta']['_product_id'][0] == $filter_by_product_id)
						array_push($new_orders_list, $order);
				}
			}
			
			$orders = $new_orders_list;
		} 
		
		return $orders;
	 
	}
	public function get_user_orders_ids($user_id, $starting_date = null, $ending_date = null, $filter_by_product_id = null, $all_types=false)
	{
		global $wpdb;
		//HPOS: custom query
		$query_string = "SELECT posts.ID 
						 FROM {$wpdb->prefix}wc_orders as orders
						 WHERE orders.customer_id = '{$user_id}'";
						 
		if(isset($starting_date) && isset($ending_date) && !empty($starting_date) && !empty($starting_date))
		{
			$query_string .= " AND orders.date_created_gmt >= '{$starting_date}'
						      and orders.date_created_gmt <= '{$ending_date}' ";
		}
	
		return $wpdb->get_results($query_string);
	}
	public function get_last_order_date($user_id)
	{
		if(!$user_id)
			return false;
		
		$version_and_statuses = $this->get_order_statuses();
		$orders=array();//order ids
		$statuses = array_keys($version_and_statuses['statuses']);
		
		$args = array(
				'limit'			=>	1,
				'orderby'       => 'date',
				'order'         => 'DESC',
				'type' 			=> 'shop_order',
				'customer_id' 	=> $user_id
		);
		
		$orders = wc_get_orders($args); 	
		if($orders && isset($orders[0]))
		{
			$date = $orders[0]->get_date_created();
			return $date->date_i18n();
		}
		return esc_html__('No orders', 'woocommerce-customers-manager' );
	}
	public function get_first_order_date($user_id)
	{
		if(!$user_id)
			return false;
		
		$args = array(
			'limit'			=>	1,
				'orderby'       => 'date',
				'order'         => 'ASC',
				'type' 			=> 'shop_order',
				'customer_id' 	=> $user_id
		);
		
		$orders = wc_get_orders($args); 	
		if($orders && isset($orders[0]))
		{
			$date = $orders[0]->get_date_created();
			return $date->date_i18n();
		}
		return esc_html__('No orders', 'woocommerce-customers-manager' );
	}
	public function get_orders_num($user_id,$starting_date = null, $ending_date = null)
	{
		if(!$user_id)
			return false;
		
		$version_and_statuses = $this->get_order_statuses();
		$tax_query = array();
		$statuses = 'publish';
		$orders=array();
		$statuses = array_keys($version_and_statuses['statuses']);
		
		$args = array(
				'limit'			=>	1,
				'orderby'       => 'date',
				'order'         => 'DESC',
				'type' 			=> 'shop_order',
				'customer_id' 	=> $user_id
		);
		if(isset($starting_date) && isset($ending_date) && !empty($starting_date) && !empty($starting_date))
		{
			list($starting_year, $starting_month, $starting_day) = explode ("/", $starting_date, 3);
			list($ending_year, $ending_month, $ending_day) = explode ("/", $ending_date, 3);
			
			$args['date_query']  = array( 'after' => array('year'  => $starting_year,
															'month' => $starting_month,
															'day'   => $starting_day,
															 'hour' => 00,
															 'minute' => 00,
															 'second' => 00
															),
										  'before' => array('year'  => $ending_year,
															'month' => $ending_month,
															'day'   => $ending_day,
															'hour' => 23,
															 'minute' => 59,
															 'second' => 59
															),
										  'inclusive' => true	
 										);
		}
		$orders = wc_get_orders($args); 	
		
		return count($orders);
	}
	//Used only for "Who bought" feature --> NO LONGER USED
	public function get_all_orders_filtered_by_date_and_or_product($product_id = null, $starting_date = null, $ending_date = null, $get_count = false , $per_page = null, $offset = null)
	{
		global $wpdb,$wccm_customer_model, $wccm_product_model;
		$wpml_join = $wpml_where =  "";
		$all_product_id = array();
		$is_variation = $wccm_product_model->is_variation($product_id) != 0 ? true : false;
		
		$post_type_for_query = $is_variation ? '_variation_id' : '_product_id';
		
		if($product_id)
			$all_product_id = $wccm_wpml_helper->get_all_product_id_translations($product_id, false, true);
		
		if(!$get_count )
			$query_string = " SELECT orders.id, postmeta.meta_value as user_id, users.user_email, users.user_registered, users.user_pass ";
		else
			$query_string = " SELECT COUNT(DISTINCT postmeta.meta_value) as total ";
			
		$query_string .= "FROM  {$wpdb->prefix}wc_orders as orders
			 INNER JOIN   {$wpdb->prefix}woocommerce_order_items AS wc_order_items ON wc_order_items.order_id = orders.id	
			 INNER JOIN   {$wpdb->prefix}woocommerce_order_itemmeta AS wc_order_item_meta ON wc_order_item_meta.order_item_id = wc_order_items.order_item_id		
			 INNER JOIN   {$wpdb->users} AS users ON users.ID = orders.customer_id
			 INNER JOIN   {$wpdb->usermeta} AS usermeta ON users.ID = usermeta.user_id
			 ".$wpml_join."
   			 WHERE       orders.type = 'shop_order'
			 AND         usermeta.meta_key = '{$wpdb->prefix}capabilities'
			     		 {$wccm_customer_model->get_user_role_list_for_sql_query()}
			 AND 		 wc_order_item_meta.meta_key = '{$post_type_for_query}' 
			 AND         wc_order_items.order_item_type IN ( 'line_item' ) ";
			
			
			
	
		if(isset($starting_date) && isset($ending_date) && !empty($ending_date) && !empty($starting_date))
		{
			$query_string .= " AND orders.date_created_gmt >= '{$starting_date} 00:00:00'
						      AND orders.date_created_gmt <= '{$ending_date} 23:59:59' ";
		}
		if($product_id)
		{
			array_push($all_product_id, $product_id);
			$query_string .= " AND (wc_order_item_meta.meta_value IN ('".implode("','",$all_product_id)."'))";
		} 
		
		if(!$get_count && $product_id)
			$query_string .=  " GROUP BY user_id";
		elseif(!$get_count)
			$query_string .=  " GROUP BY posts.ID";
		
		if(isset($per_page) && isset($offset) && !$get_count)
		{
			$query_string .=  " LIMIT {$offset},{$per_page} "; 
		}
		
		$wpdb->query('SET SQL_BIG_SELECTS=1');
		if($get_count)
		{
			$result = $wpdb->get_col( $query_string ); 
			return $result[0]; 
		}
		$line_items = $wpdb->get_results( $query_string ); 	
		return $line_items;
	}
}	
?>